@@ -3,24 +3,22 @@ from cms.models import Page
 from cms.models.titlemodels import Title
 from cms.utils import i18n
 from collections import defaultdict
+from cms.utils.conf import get_cms_setting
 from django.conf import settings
 from django.contrib.sites.models import Site
 from django.core.cache import cache
 from django.db.models.signals import post_save, post_delete
-from django.utils import translation
 from django.utils.datastructures import SortedDict
 from django.utils.safestring import mark_safe
 
 
 def update_site_and_page_choices(lang=None):
-    lang = lang or translation.get_language()
+    lang = lang or i18n.get_current_language()
     SITE_CHOICES_KEY = get_site_cache_key(lang)
     PAGE_CHOICES_KEY = get_page_cache_key(lang)
-    if settings.CMS_MODERATOR:
-        title_queryset = Title.objects.filter(page__publisher_is_draft=False)
-    else:
-        title_queryset = Title.objects.filter(page__publisher_is_draft=True)
-    title_queryset = title_queryset.select_related('page', 'page__site').order_by('page__tree_id', 'page__lft', 'page__rght')
+    title_queryset = (Title.objects.drafts()
+                      .select_related('page', 'page__site')
+                      .order_by('page__tree_id', 'page__lft', 'page__rght'))
     pages = defaultdict(SortedDict)
     sites = {}
     for title in title_queryset:
@@ -59,14 +57,14 @@ def update_site_and_page_choices(lang=None):
     return site_choices, page_choices
 
 def get_site_choices(lang=None):
-    lang = lang or translation.get_language()
+    lang = lang or i18n.get_current_language()
     site_choices = cache.get(get_site_cache_key(lang))
     if site_choices is None:
         site_choices, page_choices = update_site_and_page_choices(lang)
     return site_choices
 
 def get_page_choices(lang=None):
-    lang = lang or translation.get_language()
+    lang = lang or i18n.get_current_language()
     page_choices = cache.get(get_page_cache_key(lang))
     if page_choices is None:
         site_choices, page_choices = update_site_and_page_choices(lang)
@@ -76,22 +74,22 @@ def _get_key(prefix, lang):
     return "%s-%s" % (prefix, lang)
 
 def get_site_cache_key(lang):
-    return _get_key(settings.CMS_SITE_CHOICES_CACHE_KEY, lang)
+    return _get_key(get_cms_setting('SITE_CHOICES_CACHE_KEY'), lang)
 
 def get_page_cache_key(lang):
-    return _get_key(settings.CMS_PAGE_CHOICES_CACHE_KEY, lang)
+    return _get_key(get_cms_setting('PAGE_CHOICES_CACHE_KEY'), lang)
 
 def _clean_many(prefix):
     keys = []
-    for lang in [l[0] for l in settings.LANGUAGES]:
+    for lang in [language[0] for language in settings.LANGUAGES]:
         keys.append(_get_key(prefix, lang))
     cache.delete_many(keys)
 
 def clean_site_choices_cache(sender, **kwargs):
-    _clean_many(settings.CMS_SITE_CHOICES_CACHE_KEY)
+    _clean_many(get_cms_setting('SITE_CHOICES_CACHE_KEY'))
 
 def clean_page_choices_cache(sender, **kwargs):
-    _clean_many(settings.CMS_PAGE_CHOICES_CACHE_KEY)
+    _clean_many(get_cms_setting('PAGE_CHOICES_CACHE_KEY'))
 
 post_save.connect(clean_page_choices_cache, sender=Page)
 post_save.connect(clean_site_choices_cache, sender=Site)
