@@ -9,8 +9,6 @@ from cms.publisher.options import PublisherOptions
 class PageMetaClass(MPTTModelBase):
     def __new__(cls, name, bases, attrs):
         super_new = super(PageMetaClass, cls).__new__
-        if not settings.CMS_MODERATOR:
-            return super_new(cls, name, bases, attrs)
 
         if 'objects' in attrs:
             if not isinstance(attrs['objects'], PublisherManager):
