@@ -3,7 +3,9 @@ from classytags.arguments import Argument
 from classytags.core import Options, Tag
 from classytags.helpers import InclusionTag
 from cms.models import MASK_PAGE, MASK_CHILDREN, MASK_DESCENDANTS
+from cms.utils import get_cms_setting
 from cms.utils.admin import get_admin_menu_item_context
+from cms.utils.i18n import get_language_object
 from cms.utils.permissions import get_any_page_view_permissions
 from distutils.version import LooseVersion
 from django import template
@@ -116,11 +118,11 @@ register.tag(CleanAdminListFilter)
 @register.filter
 def boolean_icon(value):
     BOOLEAN_MAPPING = {True: 'yes', False: 'no', None: 'unknown'}
-    return mark_safe(u'<img src="%sicon-%s.gif" alt="%s" />' % (CMS_ADMIN_ICON_BASE, BOOLEAN_MAPPING[value], value))
+    return mark_safe(u'<img src="%sicon-%s.gif" alt="%s" />' % (CMS_ADMIN_ICON_BASE, BOOLEAN_MAPPING.get(value, 'unknown'), value))
 
 @register.filter
 def is_restricted(page, request):
-    if settings.CMS_PERMISSION:
+    if get_cms_setting('PERMISSION'):
         all_perms = list(get_any_page_view_permissions(request, page))
         icon = boolean_icon(bool(all_perms))
         return mark_safe(
@@ -136,38 +138,17 @@ def is_restricted(page, request):
                 'icon': icon,
             })
 
-@register.filter
-def moderator_choices(page, user):    
-    """Returns simple moderator choices used for checkbox rendering, as a value
-    is used mask value. Optimized, uses caching from change list.
-    """
-    moderation_value = page.get_moderation_value(user)
-    
-    moderate = (
-        (MASK_PAGE, _('Moderate page'), _('Unbind page moderation'), 'page'), 
-        (MASK_CHILDREN, _('Moderate children'), _('Unbind children moderation'), 'children'),
-        (MASK_DESCENDANTS, _('Moderate descendants'), _('Unbind descendants moderation'), 'descendants'),
-    )
-    
-    choices = []
-    for mask_value, title_yes, title_no, kind in moderate:
-        active = moderation_value and moderation_value & mask_value
-        title = active and title_no or title_yes
-        choices.append((mask_value, title, active, kind))
-    
-    return choices
 
 @register.filter
 def preview_link(page, language):
-    if 'cms.middleware.multilingual.MultilingualURLMiddleware' in settings.MIDDLEWARE_CLASSES:
+    if settings.USE_I18N:
         from django.core.urlresolvers import reverse
 
         # Which one of page.get_slug() and page.get_path() is the right
         # one to use in this block? They both seem to return the same thing.
         try:
             # attempt to retrieve the localized path/slug and return
-            root = reverse('pages-root')
-            return "/" + language + root + page.get_absolute_url(language, fallback=False)[len(root):]
+            return page.get_absolute_url(language, fallback=False)
         except:
             # no localized path/slug. therefore nothing to preview. stay on the same page.
             # perhaps the user should be somehow notified for this.
@@ -210,7 +191,7 @@ class PageSubmitRow(InclusionTag):
             'is_popup': is_popup,
             'show_save': True,
             'language': language,
-            'language_name': [name for langcode, name in settings.CMS_LANGUAGES if langcode == language][0],
+            'language_name': get_language_object(language)['name'],
             'show_delete_translation': show_delete_translation
         }
 register.tag(PageSubmitRow)
