@@ -1,20 +1,22 @@
 # -*- coding: utf-8 -*-
 from cms.models import Page
-from cms.test_utils.util.context_managers import (UserLoginContext, 
+from cms.test_utils.util.context_managers import (UserLoginContext,
     SettingsOverride)
 from django.conf import settings
-from django.contrib.auth.models import User, AnonymousUser, Permission
+from django.contrib.auth.models import User, AnonymousUser
 from django.contrib.sites.models import Site
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.urlresolvers import reverse
 from django.template.context import Context
 from django.test import testcases
-from django.test.client import Client, RequestFactory
+from django.test.client import RequestFactory
+from django.utils.translation import activate
 from menus.menu_pool import menu_pool
 from urlparse import urljoin
 import sys
 import urllib
 import warnings
+from cms.utils.permissions import set_current_user
 
 
 URL_CMS_PAGE = "/en/admin/cms/page/"
@@ -24,9 +26,12 @@ URL_CMS_PAGE_CHANGE_LANGUAGE = URL_CMS_PAGE_CHANGE + "?language=%s"
 URL_CMS_PAGE_DELETE = urljoin(URL_CMS_PAGE_CHANGE, "delete/")
 URL_CMS_PLUGIN_ADD = urljoin(URL_CMS_PAGE_CHANGE, "add-plugin/")
 URL_CMS_PLUGIN_EDIT = urljoin(URL_CMS_PAGE_CHANGE, "edit-plugin/")
+URL_CMS_PLUGIN_MOVE = urljoin(URL_CMS_PAGE_CHANGE, "move-plugin/")
 URL_CMS_PLUGIN_REMOVE = urljoin(URL_CMS_PAGE_CHANGE, "remove-plugin/")
 URL_CMS_TRANSLATION_DELETE = urljoin(URL_CMS_PAGE_CHANGE, "delete-translation/")
-URL_CMS_PAGE_CHANGE_STATUS = urljoin(URL_CMS_PAGE_CHANGE, "change-status/")
+
+URL_CMS_PAGE_HISTORY = urljoin(URL_CMS_PAGE_CHANGE, "history/%d/")
+URL_CMS_PLUGIN_HISTORY_EDIT = urljoin(URL_CMS_PAGE_HISTORY, "edit-plugin/")
 
 
 class _Warning(object):
@@ -74,7 +79,8 @@ class CMSTestCase(testcases.TestCase):
     def _fixture_setup(self):
         super(CMSTestCase, self)._fixture_setup()
         self.create_fixtures()
-        self.client = Client()
+        activate("en")
+
 
     def create_fixtures(self):
         pass
@@ -83,60 +89,29 @@ class CMSTestCase(testcases.TestCase):
         # Needed to clean the menu keys cache, see menu.menu_pool.clear()
         menu_pool.clear()
         super(CMSTestCase, self)._post_teardown()
+        set_current_user(None)
 
     def login_user_context(self, user):
         return UserLoginContext(self, user)
 
-    def _create_user(self, username, is_staff=False, is_superuser=False,
-                     is_active=True, add_default_permissions=False, permissions=None):
-        """
-        Use this method to create users.
-
-        Default permissions on page and text plugin are added if creating a
-        non-superuser and `add_default_permissions` is set.
-
-        Set `permissions` parameter to an iterable of permission codes to add
-        custom permissios.
-        """
-        user = User(username=username, email=username+'@django-cms.org',
-                    is_staff=is_staff, is_active=is_active, is_superuser=is_superuser)
-        user.set_password(username)
-        user.save()
-        if is_staff and not is_superuser and add_default_permissions:
-            user.user_permissions.add(Permission.objects.get(codename='add_text'))
-            user.user_permissions.add(Permission.objects.get(codename='delete_text'))
-            user.user_permissions.add(Permission.objects.get(codename='change_text'))
-
-            user.user_permissions.add(Permission.objects.get(codename='add_page'))
-            user.user_permissions.add(Permission.objects.get(codename='change_page'))
-            user.user_permissions.add(Permission.objects.get(codename='delete_page'))
-        if is_staff and not is_superuser and permissions:
-            for permission in permissions:
-                user.user_permissions.add(Permission.objects.get(codename=permission))
-        return user
-
     def get_superuser(self):
         try:
             admin = User.objects.get(username="admin")
         except User.DoesNotExist:
-            admin = self._create_user("admin", is_staff=True, is_superuser=True)
+            admin = User(username="admin", is_staff=True, is_active=True, is_superuser=True)
+            admin.set_password("admin")
+            admin.save()
         return admin
 
     def get_staff_user_with_no_permissions(self):
         """
         Used in security tests
         """
-        staff = self._create_user("staff", is_staff=True, is_superuser=False)
+        staff = User(username="staff", is_staff=True, is_active=True)
+        staff.set_password("staff")
+        staff.save()
         return staff
 
-    def get_staff_user_with_std_permissions(self):
-        """
-        This is a non superuser staff
-        """
-        staff = self._create_user("staff", is_staff=True, is_superuser=False,
-                                  add_permissions=True)
-        return staff
-    
     def get_new_page_data(self, parent_id=''):
         page_data = {
             'title': 'test page %d' % self.counter,
@@ -155,6 +130,7 @@ class CMSTestCase(testcases.TestCase):
         page_data['pagepermission_set-2-MAX_NUM_FORMS'] = 0
         self.counter = self.counter + 1
         return page_data
+
     
     def get_new_page_data_dbfields(self, parent=None, site=None,
                                    language=None,
@@ -170,6 +146,7 @@ class CMSTestCase(testcases.TestCase):
         self.counter = self.counter + 1
         return page_data
     
+    
     def get_pagedata_from_dbfields(self, page_data):
         """Converts data created by get_new_page_data_dbfields to data
         created from get_new_page_data so you can switch between test cases
@@ -185,6 +162,7 @@ class CMSTestCase(testcases.TestCase):
         page_data['pagepermission_set-2-MAX_NUM_FORMS'] = 0
         return page_data
     
+
     def print_page_structure(self, qs):
         """Just a helper to see the page struct.
         """
@@ -208,14 +186,14 @@ class CMSTestCase(testcases.TestCase):
             return qs.get(**filter)
         except ObjectDoesNotExist:
             pass
-        raise self.failureException, "ObjectDoesNotExist raised"
+        raise self.failureException, "ObjectDoesNotExist raised for filter %s" % filter
 
     def assertObjectDoesNotExist(self, qs, **filter):
         try:
             qs.get(**filter)
         except ObjectDoesNotExist:
             return
-        raise self.failureException, "ObjectDoesNotExist not raised"
+        raise self.failureException, "ObjectDoesNotExist not raised for filter %s" % filter
 
     def copy_page(self, page, target_page):
         from cms.utils.page import get_available_slug
@@ -232,7 +210,7 @@ class CMSTestCase(testcases.TestCase):
         self.assertEquals(response.status_code, 200)
         # Altered to reflect the new django-js jsonified response messages
         self.assertEquals(response.content, '{"status": 200, "content": "ok"}')
-        
+
         title = page.title_set.all()[0]
         copied_slug = get_available_slug(title)
 
@@ -280,6 +258,22 @@ class CMSTestCase(testcases.TestCase):
         request.user = getattr(self, 'user', AnonymousUser())
         request.LANGUAGE_CODE = language
         request._dont_enforce_csrf_checks = not enforce_csrf_checks
+
+        class MockStorage(object):
+
+            def __len__(self):
+                return 0
+
+            def __iter__(self):
+                return iter([])
+
+            def add(self, level, message, extra_tags=''):
+                pass
+
+            def update(self, response):
+                pass
+
+        request._messages = MockStorage()
         return request
 
     def check_published_page_attributes(self, page):
@@ -305,21 +299,6 @@ class CMSTestCase(testcases.TestCase):
             self.assertEqual(sibling.id,
                 public_siblings[i - skip].publisher_draft.id)
 
-    def request_moderation(self, page, level):
-        """Assign current logged in user to the moderators / change moderation
-
-        Args:
-            page: Page on which moderation should be changed
-
-            level <0, 7>: Level of moderation,
-                1 - moderate page
-                2 - moderate children
-                4 - moderate descendants
-                + combinations
-        """
-        response = self.client.post("/admin/cms/page/%d/change-moderation/" % page.id, {'moderate': level})
-        self.assertEquals(response.status_code, 200)
-
     def failUnlessWarns(self, category, message, f, *args, **kwargs):
         warningsShown = []
         result = _collectWarnings(warningsShown.append, f, *args, **kwargs)
