@@ -6,15 +6,14 @@ from cms.tests.api import *
 from cms.tests.apphooks import *
 from cms.tests.docs import *
 from cms.tests.forms import *
+from cms.tests.i18n import *
 from cms.tests.mail import *
 from cms.tests.menu import *
 from cms.tests.menu_utils import *
-from cms.tests.middleware import *
 from cms.tests.multilingual import *
 from cms.tests.navextender import *
 from cms.tests.nonroot import *
 from cms.tests.page import *
-from cms.tests.permissions import *
 from cms.tests.permmod import *
 from cms.tests.placeholder import *
 from cms.tests.plugins import *
@@ -25,12 +24,13 @@ from cms.tests.reversion_tests import *
 from cms.tests.security import *
 from cms.tests.settings import *
 from cms.tests.site import *
+from cms.tests.staticfiles import *
 from cms.tests.templatetags import *
 from cms.tests.toolbar import *
 from cms.tests.urlutils import *
 from cms.tests.views import *
 from cms.tests.management import *
-from cms.tests.fixture_loading import *
 from cms.tests.menu_page_viewperm import *
 from cms.tests.menu_page_viewperm_staff import *
 from cms.tests.nested_plugins import *
+from cms.tests.check import *
