@@ -1,8 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import with_statement
-from django.utils.numberformat import format
+from cms import constants
 from cms.api import add_plugin, create_page
-from cms.conf.global_settings import CMS_TEMPLATE_INHERITANCE_MAGIC
 from cms.exceptions import DuplicatePlaceholderWarning
 from cms.models.fields import PlaceholderField
 from cms.models.placeholdermodel import Placeholder
@@ -13,10 +12,10 @@ from cms.plugins.text.cms_plugins import TextPlugin
 from cms.plugins.text.models import Text
 from cms.test_utils.fixtures.fakemlng import FakemlngFixtures
 from cms.test_utils.project.fakemlng.models import Translations
-from cms.test_utils.project.placeholderapp.models import (Example1, Example2, 
-    Example3, Example4, Example5)
+from cms.test_utils.project.placeholderapp.models import (Example1, Example2,
+    Example3, Example4, Example5, MultilingualExample1)
 from cms.test_utils.testcases import CMSTestCase
-from cms.test_utils.util.context_managers import (SettingsOverride, 
+from cms.test_utils.util.context_managers import (SettingsOverride,
     UserLoginContext)
 from cms.test_utils.util.mock import AttributeObject
 from cms.utils.placeholder import PlaceholderNoAction, MLNGPlaceholderActions
@@ -34,52 +33,69 @@ from django.test import TestCase
 
 class PlaceholderTestCase(CMSTestCase):
     def setUp(self):
-        u = User(username="test", is_staff = True, is_active = True, is_superuser = True)
+        u = User(username="test", is_staff=True, is_active=True, is_superuser=True)
         u.set_password("test")
         u.save()
-        
+
         self._login_context = self.login_user_context(u)
         self._login_context.__enter__()
-    
+
     def tearDown(self):
         self._login_context.__exit__(None, None, None)
-        
+
     def test_placeholder_scanning_extend(self):
         placeholders = get_placeholders('placeholder_tests/test_one.html')
         self.assertEqual(sorted(placeholders), sorted([u'new_one', u'two', u'three']))
-        
+
+    def test_placeholder_scanning_sekizai_extend(self):
+        placeholders = get_placeholders('placeholder_tests/test_one_sekizai.html')
+        self.assertEqual(sorted(placeholders), sorted([u'new_one', u'two', u'three']))
+
     def test_placeholder_scanning_include(self):
         placeholders = get_placeholders('placeholder_tests/test_two.html')
         self.assertEqual(sorted(placeholders), sorted([u'child', u'three']))
-        
+
     def test_placeholder_scanning_double_extend(self):
         placeholders = get_placeholders('placeholder_tests/test_three.html')
         self.assertEqual(sorted(placeholders), sorted([u'new_one', u'two', u'new_three']))
-        
+
+    def test_placeholder_scanning_sekizai_double_extend(self):
+        placeholders = get_placeholders('placeholder_tests/test_three_sekizai.html')
+        self.assertEqual(sorted(placeholders), sorted([u'new_one', u'two', u'new_three']))
+
     def test_placeholder_scanning_complex(self):
         placeholders = get_placeholders('placeholder_tests/test_four.html')
         self.assertEqual(sorted(placeholders), sorted([u'new_one', u'child', u'four']))
-        
+
     def test_placeholder_scanning_super(self):
         placeholders = get_placeholders('placeholder_tests/test_five.html')
         self.assertEqual(sorted(placeholders), sorted([u'one', u'extra_one', u'two', u'three']))
-        
+
     def test_placeholder_scanning_nested(self):
         placeholders = get_placeholders('placeholder_tests/test_six.html')
         self.assertEqual(sorted(placeholders), sorted([u'new_one', u'new_two', u'new_three']))
-        
+
     def test_placeholder_scanning_duplicate(self):
-        placeholders = self.assertWarns(DuplicatePlaceholderWarning, "Duplicate placeholder found: `one`", get_placeholders, 'placeholder_tests/test_seven.html')
+        placeholders = self.assertWarns(DuplicatePlaceholderWarning, "Duplicate placeholder found: `one`",
+                                        get_placeholders, 'placeholder_tests/test_seven.html')
         self.assertEqual(sorted(placeholders), sorted([u'one']))
 
     def test_placeholder_scanning_extend_outside_block(self):
         placeholders = get_placeholders('placeholder_tests/outside.html')
         self.assertEqual(sorted(placeholders), sorted([u'new_one', u'two', u'base_outside']))
 
+    def test_placeholder_scanning_sekizai_extend_outside_block(self):
+        placeholders = get_placeholders('placeholder_tests/outside_sekizai.html')
+        self.assertEqual(sorted(placeholders), sorted([u'new_one', u'two', u'base_outside']))
+
     def test_placeholder_scanning_extend_outside_block_nested(self):
         placeholders = get_placeholders('placeholder_tests/outside_nested.html')
         self.assertEqual(sorted(placeholders), sorted([u'new_one', u'two', u'base_outside']))
-    
+
+    def test_placeholder_scanning_sekizai_extend_outside_block_nested(self):
+        placeholders = get_placeholders('placeholder_tests/outside_nested_sekizai.html')
+        self.assertEqual(sorted(placeholders), sorted([u'new_one', u'two', u'base_outside']))
+
     def test_fieldsets_requests(self):
         response = self.client.get(reverse('admin:placeholderapp_example1_add'))
         self.assertEqual(response.status_code, 200)
@@ -91,9 +107,10 @@ class PlaceholderTestCase(CMSTestCase):
         self.assertEqual(response.status_code, 200)
         response = self.client.get(reverse('admin:placeholderapp_example5_add'))
         self.assertEqual(response.status_code, 200)
-        
+
     def test_fieldsets(self):
         from cms.test_utils.project.placeholderapp import admin as __ # load admin
+
         request = self.get_request('/')
         admins = [
             (Example1, 2),
@@ -110,7 +127,7 @@ class PlaceholderTestCase(CMSTestCase):
             self.assertEqual(len(fieldsets), fscount, (
                 "Asserting fieldset count for %s. Got %s instead of %s: %s. "
                 "Using %s." % (model.__name__, len(fieldsets),
-                    fscount, fieldsets, ainstance.__class__.__name__)      
+                fscount, fieldsets, ainstance.__class__.__name__)
             ))
             for label, fieldset in fieldsets:
                 fields = list(fieldset['fields'])
@@ -121,7 +138,7 @@ class PlaceholderTestCase(CMSTestCase):
                         self.assertTrue('plugin-holder-nopage' in fieldset['classes'])
                         phfields.remove(field)
             self.assertEqual(phfields, [])
-            
+
     def test_page_only_plugins(self):
         ex = Example1(
             char_1='one',
@@ -133,7 +150,7 @@ class PlaceholderTestCase(CMSTestCase):
         response = self.client.get(reverse('admin:placeholderapp_example1_change', args=(ex.pk,)))
         self.assertEqual(response.status_code, 200)
         self.assertNotContains(response, 'InheritPagePlaceholderPlugin')
-        
+
     def test_inter_placeholder_plugin_move(self):
         ex = Example5(
             char_1='one',
@@ -167,7 +184,7 @@ class PlaceholderTestCase(CMSTestCase):
         containing an hyphen, the hyphen is escaped by django escapejs resulting
         in a incorrect URL
         """
-        with SettingsOverride(CMS_MODERATOR=False, CMS_PERMISSION=False):
+        with SettingsOverride(CMS_PERMISSION=False):
             ex = Example1(
                 char_1='one',
                 char_2='two',
@@ -185,14 +202,14 @@ class PlaceholderTestCase(CMSTestCase):
                 reverse('admin:placeholderapp_example1_change', args=(ex.pk,)),
                 test_plugin.pk)
             response = self.client.post(pl_url, {})
-            self.assertContains(response,"/static/plugins/empty-image-file.png")
+            self.assertContains(response, "/static/plugins/empty-image-file.png")
 
     def test_nested_plugin_escapejs_page(self):
         """
         Sibling test of the above, on a page.
         #1366 does not apply to placeholder defined in a page
         """
-        with SettingsOverride(CMS_MODERATOR=False, CMS_PERMISSION=False):
+        with SettingsOverride(CMS_PERMISSION=False):
             page = create_page('page', 'col_two.html', 'en')
             ph1 = page.placeholders.get(slot='col_left')
             ###
@@ -204,7 +221,7 @@ class PlaceholderTestCase(CMSTestCase):
                 reverse('admin:cms_page_change', args=(page.pk,)),
                 test_plugin.pk)
             response = self.client.post(pl_url, {})
-            self.assertContains(response,"/static/plugins/empty-image-file.png")
+            self.assertContains(response, "/static/plugins/empty-image-file.png")
 
     def test_placeholder_scanning_fail(self):
         self.assertRaises(TemplateSyntaxError, get_placeholders, 'placeholder_tests/test_eleven.html')
@@ -226,14 +243,17 @@ class PlaceholderTestCase(CMSTestCase):
         placeholder = self.reload(placeholder)
         rctx['placeholder'] = placeholder
         self.assertEqual(template.render(rctx).strip(), "test")
-    
+
     def test_placeholder_context_leaking(self):
         TEST_CONF = {'test': {'extra_context': {'width': 10}}}
         ph = Placeholder.objects.create(slot='test')
+
         class NoPushPopContext(Context):
             def push(self):
                 pass
+
             pop = push
+
         context = NoPushPopContext()
         context['request'] = self.get_request()
         with SettingsOverride(CMS_PLACEHOLDER_CONF=TEST_CONF):
@@ -251,37 +271,13 @@ class PlaceholderTestCase(CMSTestCase):
     def test_placeholder_field_no_related_name(self):
         self.assertRaises(ValueError, PlaceholderField, 'placeholder', related_name='+')
 
-    def test_placeholder_pk_thousands_format(self):
-        page = create_page("page", "nav_playground.html", "en", published=True)
-        page.publish()
-        page = page.get_public_object()
-        for placeholder in page.placeholders.all():
-            page.placeholders.remove(placeholder)
-            placeholder.pk += 1000
-            placeholder.save()
-            page.placeholders.add(placeholder)
-        page.reload()
-        for placeholder in page.placeholders.all():
-            plugin = add_plugin(placeholder, "TextPlugin", "en", body="body",
-                                id=placeholder.pk)
-        with SettingsOverride(USE_THOUSAND_SEPARATOR=True, USE_L10N=True):
-            # Superuser
-            user = self.get_superuser()
-            self.client.login(username=user.username, password=user.username)
-            response = self.client.get("/en/?edit")
-            for placeholder in page.placeholders.all():
-                self.assertContains(response, "'placeholder_id': '%s'" % placeholder.pk)
-                self.assertNotContains(response, "'placeholder_id': '%s'" % format(
-                    placeholder.pk, ".", decimal_pos=None, grouping=3, thousand_sep=","))
-
 
 class PlaceholderActionTests(FakemlngFixtures, CMSTestCase):
-    
     def test_placeholder_no_action(self):
         actions = PlaceholderNoAction()
         self.assertEqual(actions.get_copy_languages(), [])
         self.assertFalse(actions.copy())
-        
+
     def test_mlng_placeholder_actions_get_copy_languages(self):
         actions = MLNGPlaceholderActions()
         fr = Translations.objects.get(language_code='fr')
@@ -302,39 +298,39 @@ class PlaceholderActionTests(FakemlngFixtures, CMSTestCase):
         self.assertEqual(fr_copy_languages, [EN])
         self.assertEqual(de_copy_languages, [EN, FR])
         self.assertEqual(en_copy_languages, [FR])
-        
+
     def test_mlng_placeholder_actions_copy(self):
         actions = MLNGPlaceholderActions()
         fr = Translations.objects.get(language_code='fr')
         de = Translations.objects.get(language_code='de')
         self.assertEqual(fr.placeholder.cmsplugin_set.count(), 1)
         self.assertEqual(de.placeholder.cmsplugin_set.count(), 0)
-        
+
         new_plugins = actions.copy(de.placeholder, 'fr', 'placeholder', Translations, 'de')
         self.assertEqual(len(new_plugins), 1)
-        
+
         de = self.reload(de)
         fr = self.reload(fr)
-        
+
         self.assertEqual(fr.placeholder.cmsplugin_set.count(), 1)
         self.assertEqual(de.placeholder.cmsplugin_set.count(), 1)
-        
+
     def test_mlng_placeholder_actions_empty_copy(self):
         actions = MLNGPlaceholderActions()
         fr = Translations.objects.get(language_code='fr')
         de = Translations.objects.get(language_code='de')
         self.assertEqual(fr.placeholder.cmsplugin_set.count(), 1)
         self.assertEqual(de.placeholder.cmsplugin_set.count(), 0)
-        
+
         new_plugins = actions.copy(fr.placeholder, 'de', 'placeholder', Translations, 'fr')
         self.assertEqual(len(new_plugins), 0)
 
         de = self.reload(de)
         fr = self.reload(fr)
-        
+
         self.assertEqual(fr.placeholder.cmsplugin_set.count(), 1)
         self.assertEqual(de.placeholder.cmsplugin_set.count(), 0)
-        
+
     def test_mlng_placeholder_actions_no_placeholder(self):
         actions = MLNGPlaceholderActions()
         Translations.objects.filter(language_code='nl').update(placeholder=None)
@@ -342,54 +338,55 @@ class PlaceholderActionTests(FakemlngFixtures, CMSTestCase):
         nl = Translations.objects.get(language_code='nl')
         self.assertEqual(nl.placeholder, None)
         self.assertEqual(de.placeholder.cmsplugin_set.count(), 0)
-        
+
         okay = actions.copy(de.placeholder, 'nl', 'placeholder', Translations, 'de')
         self.assertEqual(okay, False)
-        
+
         de = self.reload(de)
         nl = self.reload(nl)
-        
+
         nl = Translations.objects.get(language_code='nl')
         de = Translations.objects.get(language_code='de')
-        
+
+
 class PlaceholderModelTests(CMSTestCase):
     def get_mock_user(self, superuser):
         return AttributeObject(
             is_superuser=superuser,
             has_perm=lambda string: False,
-        ) 
-    
+        )
+
     def get_mock_request(self, superuser=True):
         return AttributeObject(
             superuser=superuser,
             user=self.get_mock_user(superuser)
         )
-    
+
     def test_check_placeholder_permissions_ok_for_superuser(self):
         ph = Placeholder.objects.create(slot='test', default_width=300)
         result = ph.has_change_permission(self.get_mock_request(True))
         self.assertTrue(result)
-        
+
     def test_check_placeholder_permissions_nok_for_user(self):
         ph = Placeholder.objects.create(slot='test', default_width=300)
         result = ph.has_change_permission(self.get_mock_request(False))
         self.assertFalse(result)
-    
+
     def test_check_unicode_rendering(self):
         ph = Placeholder.objects.create(slot='test', default_width=300)
         result = unicode(ph)
-        self.assertEqual(result,u'test')
-    
+        self.assertEqual(result, u'test')
+
     def test_excercise_get_attached_model(self):
         ph = Placeholder.objects.create(slot='test', default_width=300)
         result = ph._get_attached_model()
         self.assertEqual(result, None) # Simple PH - no model
-        
+
     def test_excercise_get_attached_field_name(self):
         ph = Placeholder.objects.create(slot='test', default_width=300)
         result = ph._get_attached_field_name()
         self.assertEqual(result, None) # Simple PH - no field name
-    
+
     def test_excercise_get_attached_models_notplugins(self):
         ex = Example1(
             char_1='one',
@@ -404,7 +401,7 @@ class PlaceholderModelTests(CMSTestCase):
         add_plugin(ph, TextPlugin, 'en', body='en body')
         result = list(ph._get_attached_models())
         self.assertEqual(result, [Example1]) # Simple PH still one Example1 model
-        
+
     def test_excercise_get_attached_fields_notplugins(self):
         ex = Example1(
             char_1='one',
@@ -419,25 +416,29 @@ class PlaceholderModelTests(CMSTestCase):
         add_plugin(ph, TextPlugin, 'en', body='en body')
         result = [f.name for f in list(ph._get_attached_fields())]
         self.assertEqual(result, ['placeholder']) # Simple PH - still one placeholder field name
-        
-class PlaceholderAdminTest(CMSTestCase):
-    placeholderconf = {'test': {
-            'limits': {
-                'global': 2,
-                'TextPlugin': 1,
-            }
-        }
-    }
+
+
+class PlaceholderAdminTestBase(CMSTestCase):
     def get_placeholder(self):
         return Placeholder.objects.create(slot='test')
-    
+
     def get_admin(self):
         admin.autodiscover()
         return admin.site._registry[Example1]
-    
+
     def get_post_request(self, data):
         return self.get_request(post_data=data)
-    
+
+
+class PlaceholderAdminTest(PlaceholderAdminTestBase):
+    placeholderconf = {'test': {
+        'limits': {
+            'global': 2,
+            'TextPlugin': 1,
+        }
+    }
+    }
+
     def test_global_limit(self):
         placeholder = self.get_placeholder()
         admin = self.get_admin()
@@ -456,7 +457,7 @@ class PlaceholderAdminTest(CMSTestCase):
                 self.assertEqual(response.status_code, 200)
                 response = admin.add_plugin(request) # third
                 self.assertEqual(response.status_code, 400)
-                self.assertEqual(response.content, "This placeholder already has the maximum number of plugins.")
+                self.assertEqual(response.content, "This placeholder already has the maximum number of plugins (2).")
 
     def test_type_limit(self):
         placeholder = self.get_placeholder()
@@ -474,13 +475,97 @@ class PlaceholderAdminTest(CMSTestCase):
                 self.assertEqual(response.status_code, 200)
                 response = admin.add_plugin(request) # second
                 self.assertEqual(response.status_code, 400)
-                self.assertEqual(response.content, "This placeholder already has the maximum number (1) of TextPlugin plugins.")
+                self.assertEqual(response.content,
+                                 "This placeholder already has the maximum number (1) of allowed Text plugins.")
+
+    def test_global_limit_on_plugin_move(self):
+        admin = self.get_admin()
+        superuser = self.get_superuser()
+        source_placeholder = Placeholder.objects.create(slot='source')
+        target_placeholder = self.get_placeholder()
+        data = {
+            'placeholder': source_placeholder,
+            'plugin_type': 'LinkPlugin',
+            'language': 'en',
+        }
+        plugin_1 = add_plugin(**data)
+        plugin_2 = add_plugin(**data)
+        plugin_3 = add_plugin(**data)
+        with UserLoginContext(self, superuser):
+            with SettingsOverride(CMS_PLACEHOLDER_CONF=self.placeholderconf):
+                request = self.get_post_request({'placeholder_id': target_placeholder.pk, 'plugin_id': plugin_1.pk})
+                response = admin.move_plugin(request) # first
+                self.assertEqual(response.status_code, 200)
+                request = self.get_post_request({'placeholder_id': target_placeholder.pk, 'plugin_id': plugin_2.pk})
+                response = admin.move_plugin(request) # second
+                self.assertEqual(response.status_code, 200)
+                request = self.get_post_request({'placeholder_id': target_placeholder.pk, 'plugin_id': plugin_3.pk})
+                response = admin.move_plugin(request) # third
+                self.assertEqual(response.status_code, 400)
+                self.assertEqual(response.content, "This placeholder already has the maximum number of plugins (2).")
+
+    def test_type_limit_on_plugin_move(self):
+        admin = self.get_admin()
+        superuser = self.get_superuser()
+        source_placeholder = Placeholder.objects.create(slot='source')
+        target_placeholder = self.get_placeholder()
+        data = {
+            'placeholder': source_placeholder,
+            'plugin_type': 'TextPlugin',
+            'language': 'en',
+        }
+        plugin_1 = add_plugin(**data)
+        plugin_2 = add_plugin(**data)
+        with UserLoginContext(self, superuser):
+            with SettingsOverride(CMS_PLACEHOLDER_CONF=self.placeholderconf):
+                request = self.get_post_request({'placeholder_id': target_placeholder.pk, 'plugin_id': plugin_1.pk})
+                response = admin.move_plugin(request) # first
+                self.assertEqual(response.status_code, 200)
+                request = self.get_post_request({'placeholder_id': target_placeholder.pk, 'plugin_id': plugin_2.pk})
+                response = admin.move_plugin(request) # second
+                self.assertEqual(response.status_code, 400)
+                self.assertEqual(response.content,
+                                 "This placeholder already has the maximum number (1) of allowed Text plugins.")
+
+    def test_edit_plugin_and_cancel(self):
+        placeholder = self.get_placeholder()
+        admin = self.get_admin()
+        data = {
+            'plugin_type': 'TextPlugin',
+            'placeholder': placeholder.pk,
+            'language': 'en',
+        }
+        superuser = self.get_superuser()
+        with UserLoginContext(self, superuser):
+            with SettingsOverride(CMS_PLACEHOLDER_CONF=self.placeholderconf):
+                request = self.get_post_request(data)
+                response = admin.add_plugin(request)
+                self.assertEqual(response.status_code, 200)
+                plugin_id = int(response.content)
+                data = {
+                    'body': 'Hello World',
+                }
+                request = self.get_post_request(data)
+                response = admin.edit_plugin(request, plugin_id)
+                self.assertEqual(response.status_code, 200)
+                text_plugin = Text.objects.get(pk=plugin_id)
+                self.assertEquals('Hello World', text_plugin.body)
+
+                # edit again, but this time press cancel
+                data = {
+                    'body': 'Hello World!!',
+                    '_cancel': True,
+                }
+                request = self.get_post_request(data)
+                response = admin.edit_plugin(request, plugin_id)
+                self.assertEqual(response.status_code, 200)
+                text_plugin = Text.objects.get(pk=plugin_id)
+                self.assertEquals('Hello World', text_plugin.body)
 
 
-class PlaceholderPluginPermissionTests(PlaceholderAdminTest):
-
+class PlaceholderPluginPermissionTests(PlaceholderAdminTestBase):
     def _testuser(self):
-        u = User(username="test", is_staff = True, is_active = True, is_superuser = False)
+        u = User(username="test", is_staff=True, is_active=True, is_superuser=False)
         u.set_password("test")
         u.save()
         return u
@@ -591,7 +676,7 @@ class PlaceholderConfTests(TestCase):
 
     def test_get_all_plugins_inherit(self):
         parent = create_page('parent', 'col_two.html', 'en')
-        page = create_page('page', CMS_TEMPLATE_INHERITANCE_MAGIC, 'en', parent=parent)
+        page = create_page('page', constants.TEMPLATE_INHERITANCE_MAGIC, 'en', parent=parent)
         placeholder = page.placeholders.get(slot='col_left')
         conf = {
             'col_two': {
@@ -605,3 +690,57 @@ class PlaceholderConfTests(TestCase):
             plugins = plugin_pool.get_all_plugins(placeholder, page)
             self.assertEqual(len(plugins), 1, plugins)
             self.assertEqual(plugins[0], LinkPlugin)
+
+
+class PlaceholderI18NTest(CMSTestCase):
+    def _testuser(self):
+        u = User(username="test", is_staff=True, is_active=True, is_superuser=True)
+        u.set_password("test")
+        u.save()
+        return u
+
+    def test_hvad_tabs(self):
+
+        ex = MultilingualExample1(
+            char_1='one',
+            char_2='two',
+        )
+        ex.save()
+        user = self._testuser()
+        self.client.login(username='test', password='test')
+
+        response = self.client.get('/de/admin/placeholderapp/multilingualexample1/1/')
+        self.assertContains(response, '<input type="hidden" class="language_button selected" name="de" />')
+
+
+    def test_no_tabs(self):
+
+        ex = Example4(
+            char_1='one',
+            char_2='two',
+            char_3='one',
+            char_4='two',
+            )
+        ex.save()
+        user = self._testuser()
+        self.client.login(username='test', password='test')
+
+        response = self.client.get('/de/admin/placeholderapp/example4/1/')
+        self.assertNotContains(response, '<input type="hidden" class="language_button selected" name="de" />')
+
+    def test_placeholder_tabs(self):
+
+        ex = Example5(
+            char_1='one',
+            char_2='two',
+            char_3='one',
+            char_4='two',
+            )
+        ex.save()
+        user = self._testuser()
+        self.client.login(username='test', password='test')
+
+        response = self.client.get('/de/admin/placeholderapp/example5/1/')
+        self.assertNotContains(response, """<input type="button" onclick="trigger_lang_button(this,'./?language=en');" class="language_button selected" id="debutton" name="en" value="English">""")
+
+
